@@ -46,9 +46,10 @@
 int nodaemon = false;
 unsigned int screen_width;
 unsigned int screen_height;
+enum mouse_reporting_mode mouse_reporting = MOUSE_REPORTING_OFF;
 
 static struct tools_options options;
-static unsigned int stop = 0;
+static volatile unsigned int stop = 0;
 static enum tools_backend backend = BACKEND_UDEV;
 static const char *seat_or_device = "seat0";
 static bool grab = false;
@@ -92,10 +93,14 @@ handle_pointer_button_event(struct libinput_event *ev)
   case BTN_MIDDLE:
     if (state==LIBINPUT_BUTTON_STATE_PRESSED)
       press_middle_button();
+    else
+      release_middle_button();
     break;
   case BTN_RIGHT:
     if (state==LIBINPUT_BUTTON_STATE_PRESSED)
       press_right_button();
+    else
+      release_right_button();
     break;
   }
 }
@@ -146,7 +151,7 @@ handle_events(struct libinput *li)
   struct libinput_event *ev;
 
   libinput_dispatch(li);
-  set_screen_size();
+  set_screen_size_and_mouse_reporting();
   while ((ev = libinput_get_event(li))) {
 
     switch (libinput_event_get_type(ev)) {
@@ -198,22 +203,11 @@ static void
 mainloop(struct libinput *li)
 {
   struct pollfd fds;
-  struct sigaction act;
 
   fds.fd = libinput_get_fd(li);
   fds.events = POLLIN;
   fds.revents = 0;
 
-  memset(&act, 0, sizeof(act));
-  act.sa_sigaction = sighandler;
-  act.sa_flags = SA_SIGINFO;
-
-  if (sigaction(SIGINT, &act, NULL) == -1) {
-    fprintf(stderr, "Failed to set up signal handling (%s)\n",
-        strerror(errno));
-    return;
-  }
-
   /* Handle already-pending device added events */
   if (handle_events(li))
     fprintf(stderr, "Expected device added events on startup but got none. "
@@ -230,6 +224,8 @@ usage(void)
          "--udev <seat>.... Use udev device discovery (default).\n"
          "                  Specifying a seat ID is optional.\n"
          "--device /path/to/device .... open the given device only\n"
+         "--apply-to=\"pattern\"....  Configuration options are only applied where the device name matches the pattern\n"
+         "--disable-sendevents=\"pattern\".... Set the send-events option to disabled for the devices matching patterns\n"
          "\n"
          "Features:\n"
          "--enable-tap\n"
@@ -367,9 +363,20 @@ int
 event_main(void)
 {
   struct libinput *li;
+  struct sigaction act;
+
+  memset(&act, 0, sizeof(act));
+  act.sa_sigaction = sighandler;
+  act.sa_flags = SA_SIGINFO;
+
+  if (sigaction(SIGINT, &act, NULL) == -1) {
+    fprintf(stderr, "Failed to set up signal handling (%s)\n",
+        strerror(errno));
+    return 0;
+  }
 
   set_lut(word_chars);
-  li = tools_open_backend(backend, seat_or_device, verbose, grab);
+  li = tools_open_backend(backend, seat_or_device, verbose, &grab);
   if (!li)
     return 1;
 
