@@ -22,6 +22,7 @@
 #include <linux/tiocl.h>
 #include <stdint.h>
 #include <linux/kd.h>
+#include <errno.h>
 
 #include "consolation.h"
 
@@ -34,14 +35,33 @@ check_mode(int fd)
 }
 
 void
-set_screen_size(void)
+set_screen_size_and_mouse_reporting(void)
 {
   struct winsize s;
+  unsigned char request = TIOCL_GETMOUSEREPORTING;
   int fd = open("/dev/tty0",O_RDONLY);
-  if (ioctl(fd, TIOCGWINSZ, &s)) perror("TIOCGWINSZ");
+  if (fd == -1)
+  {
+    perror("open /dev/tty0");
+    return;
+  }
+  if (ioctl(fd, TIOCGWINSZ, &s))
+    perror("TIOCGWINSZ");
+  else
+  {
+    screen_width  = s.ws_col;
+    screen_height = s.ws_row;
+  }
+  if (ioctl(fd, TIOCLINUX, &request))
+  {
+    perror("TIOCLINUX, TIOCL_GETMOUSEREPORTING");
+    request = MOUSE_REPORTING_OFF;
+  }
+  close(fd);
+  if (request >= MOUSE_REPORTING_MODE_COUNT)
+    request = MOUSE_REPORTING_OFF;
+  mouse_reporting = request;
   close(fd);
-  screen_width  = s.ws_col;
-  screen_height = s.ws_row;
 }
 
 static void
@@ -61,11 +81,26 @@ linux_selection(int xs, int ys, int xe, int ye, int sel_mode)
   s.sel.sel_mode = sel_mode;
   fd = open("/dev/tty0",O_RDONLY);
   if (check_mode(fd))
-    if (ioctl(fd, TIOCLINUX, ((char*)&s)+1)<0)
+  {
+    int err = ioctl(fd, TIOCLINUX, ((char*)&s)+1);
+    if (err<0 && !(errno==EINVAL && (sel_mode&TIOCL_SELMOUSEREPORT)))
+    /* The kernel return EINVAL for TIOCL_SELMOUSEREPORT when
+       TIOCL_GETMOUSEREPORTING reports 0. Unfortunately this cannot be
+       checked without race conditions, so it is simpler to ignore the
+       error.
+     */
       perror("selection: TIOCLINUX");
+  }
   close(fd);
 }
 
+void
+report_pointer(int x, int y, enum current_button button)
+{
+  linux_selection(x, y, x, y, TIOCL_SELCLEAR);
+  linux_selection(x, y, x, y, TIOCL_SELMOUSEREPORT + button );
+}
+
 void
 draw_pointer(int x, int y)
 {
