@@ -1,8 +1,8 @@
 /* Adapted from test/shared.h from the libinput distribution */
-/* test/shared.h 1.9.4:
+/* test/shared.h 1.13.4:
  *
  * Copyright © 2014 Red Hat, Inc.
- * Modifications: Copyright © 2017 Bill Allombert <ballombe@debian.org>
+ * Modifications: Copyright © 2019 Bill Allombert <ballombe@debian.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -53,6 +53,7 @@ enum configuration_options {
 	OPT_SPEED,
 	OPT_PROFILE,
 	OPT_DISABLE_SENDEVENTS,
+	OPT_APPLY_TO
 };
 
 #define CONFIGURATION_OPTIONS \
@@ -76,7 +77,8 @@ enum configuration_options {
 	{ "set-scroll-button",         required_argument, 0, OPT_SCROLL_BUTTON }, \
 	{ "set-profile",               required_argument, 0, OPT_PROFILE }, \
 	{ "set-tap-map",               required_argument, 0, OPT_TAP_MAP }, \
-	{ "set-speed",                 required_argument, 0, OPT_SPEED }
+	{ "set-speed",                 required_argument, 0, OPT_SPEED }, \
+	{ "apply-to",                  required_argument, 0, OPT_APPLY_TO }
 
 enum tools_backend {
 	BACKEND_DEVICE,
@@ -84,6 +86,8 @@ enum tools_backend {
 };
 
 struct tools_options {
+	char match[256];
+
 	int tapping;
 	int drag;
 	int drag_lock;
@@ -107,7 +111,7 @@ int tools_parse_option(int option,
 struct libinput* tools_open_backend(enum tools_backend which,
 				    const char *seat_or_device,
 				    bool verbose,
-				    bool grab);
+				    bool *grab);
 void tools_device_apply_config(struct libinput_device *device,
 			       struct tools_options *options);
 int tools_exec_command(const char *prefix, int argc, char **argv);
